using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace CreaStelline
{
    public class CreaStellineGame : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // Immagine contenente i frame dell'animazione
        Texture2D framesStellina;

        // Stato del tasto sinistro del mouse dell'ultimo
        // Update
        ButtonState lastMouseState = ButtonState.Released;
        // Generatore di numeri casuali usato durante tutto
        // il gioco
        Random gameRandom = new Random();

        // Costruttore
        public CreaStellineGame()
        {

            // Permettiamo il ridimensionamento della finestra
            this.Window.AllowUserResizing = true;

            // Mostriamo il puntatore
            this.IsMouseVisible = true;

            // Inizializziamo il GraphicsDeviceManager e impostiamo le dimensioni
            // desiderate dell'area di disegno
            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferWidth = 800;
            graphics.PreferredBackBufferHeight = 600;

            // Impostiamo la sottocartella con le risorse
            this.Content.RootDirectory = "Content";
        }

        protected override void Initialize()
        {
            // Creiamo un nuovo servizio per il disegno
            spriteBatch = new SpriteBatch(GraphicsDevice);
            this.Services.AddService(typeof(SpriteBatch), spriteBatch);

            base.Initialize();
        }

        protected override void LoadContent()
        {
            // Carichiamo l'immagine dei frame dell'animazione
            framesStellina = this.Content.Load<Texture2D>("stellina");

        }

        // Gestiamo la logica di gioco
        protected override void Update(GameTime gameTime)
        {
            if (Keyboard.GetState(PlayerIndex.One).IsKeyDown(Keys.Escape))
                this.Exit();

            // Prendiamo lo stato corrente del mouse
            MouseState mouse = Mouse.GetState();

            // Se  stato rilasciato il mouse
            if (mouse.LeftButton == ButtonState.Released && lastMouseState == ButtonState.Pressed) {

                // Creiamo l'animazione della stellina che si
                // ingrandisce
                Animation cresce = new Animation(framesStellina, 190, 25, 1);
                // Impostiamo una velocit angolare casuale
                cresce.AngularVelocity = (float) gameRandom.NextDouble() * 4 - 2;
                // Impostiamo una dimensione casuale tra le dimensioni
                // originali dell'immagine e la loro met
                cresce.Scale = (float)gameRandom.NextDouble() * 0.5f + 0.5f;

                // Cloniamo l'animazione appena creata e impostiamo
                // che vada al contrario
                Animation decresce = (Animation) ((ICloneable) cresce).Clone();
                decresce.Backward = true;
                
                // Inizializziamo la stellina
                Character stellina = new Character(this, new Vector2(mouse.X, mouse.Y));
                // Impostiamo le animazioni della stellina
                stellina.Animations = new Animation[] { cresce, decresce };

                // Aggiungiamo la stellina appena creata ai
                // componenti del gioco
                this.Components.Add(stellina);
            }

            // Aggiorniamo qual  lo stato del tasto sinistro
            lastMouseState = mouse.LeftButton;

            base.Update(gameTime);
        }

        // Gestiamo l'output grafico
        protected override void Draw(GameTime gameTime)
        {
            // Puliamo l'output grafico
            GraphicsDevice.Clear(Color.SkyBlue);

            // Iniziamo a disegnare
            this.spriteBatch.Begin();
            // Chiamiamo il metodo Draw di tutti i componenti
            base.Draw(gameTime);
            // Terminiamo il disegnare
            this.spriteBatch.End();
        }
    }
}
